#include <stdio.h>
#include <stdlib.h>
#include "bitset.h"


static void 
test (enum bitset_type type, unsigned int bits)
{
  bitset set1;
  bitset set2;
  bitset_iterator iter;
  bitset_bindex n;
  bitset_bindex i;

  if (bits < 4096)
    abort ();
  
  bitset_stats_enable ();
  
  set1 = bitset_alloc (bits, type);   
  set2 = bitset_alloc (bits, type);   
  printf ("%s\n", bitset_type_name_get (set1));

  for (i = 0; i < bits; i += 5)
    {
      bitset_set (set1, i);
      
      BITSET_FOR_EACH_REVERSE (iter, set1, n, 0)
	bitset_set (set2, n);
      
      if (!bitset_equal_p (set1, set2))
	abort ();

      BITSET_FOR_EACH (iter, set1, n, 0)
	bitset_reset (set2, n);

      if (!bitset_empty_p (set2))
	abort ();
    }


  bitset_zero (set1);
  for (i = 0; i < bits; i += 1)
    {
      bitset_bindex j;

      /* Swap seq to hammer cache.  */
      j = ((i & 0x0f) << 8) | ((i & 0x0ff0) >> 4) | (i & ~0xfff);

      bitset_set (set1, j);
      
      /* Copy SET1 to SET2 the hard way.  */
      BITSET_FOR_EACH_REVERSE (iter, set1, n, 0)
	bitset_set (set2, n);
      
      if (!bitset_equal_p (set1, set2))
	  abort ();

      /* Clear SET2 the hard way.  */
      BITSET_FOR_EACH (iter, set1, n, 0)
	bitset_reset (set2, n);

      if (!bitset_empty_p (set2))
	abort ();
    }
}  


int
main (void)
{
  int bits = 4096;

  bitset_stats_enable ();
  
  test (BITSET_ARRAY, bits);
  test (BITSET_LIST, bits);
  test (BITSET_TABLE, bits);
  test (BITSET_VARRAY, bits);

  bitset_stats_dump (stderr);
  return 0;
}
