#include <stdlib.h>
#include <bitset.h>

static void
test_ones (bitset set1, bitset set2, bitset set3)
{
  /* This test will make a sparse bitset full.  */
  bitset_ones (set1);
  
  if (bitset_count (set1) != bitset_size (set1))
      abort ();

  if (bitset_size (set1) != bitset_size (set2))
      abort ();

  printf ("%s bits %d\n", bitset_type_name_get (set1), bitset_size (set1));
}


static void
test_not (bitset set1, bitset set2, bitset set3)
{
  /* This test will make a sparse bitset full.  */
  bitset_zero (set1);
  bitset_not (set3, set1);

  bitset_not (set3, set3);
  if (!bitset_empty_p (set3))
    abort ();

  bitset_ones (set3);
  bitset_not (set3, set3);
  if (!bitset_empty_p (set3))
    abort ();
}


static void
test (enum bitset_type type, unsigned int bits)
{
  bitset set1;
  bitset set2;
  bitset set3;
    
  set1 = bitset_alloc (bits, type);
  set2 = bitset_alloc (bits, type);
  set3 = bitset_alloc (bits, type);
    
  /* The sets should be created empty.  */
  if (!bitset_empty_p (set1))
    abort ();
  if (!bitset_empty_p (set2))
    abort ();
  if (!bitset_empty_p (set3))
    abort ();

  test_ones (set1, set2, set3);
    
  test_not (set1, set2, set3);

  bitset_free (set1);
  bitset_free (set2);
  bitset_free (set3);
}


int
main (void)
{
  bitset_stats_enable ();

  test (BITSET_ARRAY, 4096 + 13);
  test (BITSET_LIST, 4096 + 13);
  test (BITSET_TABLE, 4096 + 13);
  test (BITSET_VARRAY, 4096 + 13);

  bitset_stats_dump (stderr);
  return 0;
}
